//lighting.fxh
#ifndef LIGHTING_FXH
#define LIGHTING_FXH

#ifdef GL_ES
precision mediump float;
#endif

#define float4 vec4
#define float3 vec3
#define float2 vec2
#define int4 ivec4
#define saturate(a) max(min(1.0,a),0.0)

uniform highp float4 g_aPointLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_POINTLIGHT ];
uniform highp float4 g_aDirLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_DIRLIGHT ];
uniform highp float4 g_aSpotLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_SPOTLIGHT ];
uniform highp int4 g_vLightCounts;

#define g_iNumDirLights (g_vLightCounts.x)
#define g_iNumPointLights (g_vLightCounts.y)
#define g_iNumSpotLights (g_vLightCounts.z)

uniform highp float3 g_vCameraPos;
uniform highp float3 g_vGlobalAmbient;

#define PLCOLOR( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].rgb )
#define PLINVSQRRANGE( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].w )
#define PLPOS( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1 ].xyz )
#define PLPOWER( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1].w )


#define DLCOLOR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i ].rgb )
#define DLDIR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i + 1 ].xyz )

#define SPLCOLOR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].rgb )
#define SPLINVSQRRANGE( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].w )
#define SPLPOS( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].xyz )
#define SPLANGLESUB( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].w )
#define SPLDIR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].xyz )
#define SPLANGLEMUL( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].w )
#define SPLFALLOFF( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].x )
#define SPLPOWER( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].y )


float3 BlinnPhong(float3 N, float3 L, float3 H, float3 vLightColor, float3 vDiffuse, float3 vSpecular, float vSpecularPower)
{
	float NL = saturate(dot(N, L));
	float NH = saturate(dot(N, H));
	return (pow( NH, vSpecularPower ) * vSpecular + vDiffuse) * NL * vLightColor;	//CHECK This.
}

//Point attenuation function suggested by Jorge. With pow function
float PointAttenuationEx(float fInvSqrRange, float fDistance, float fPower)
{
	return pow(saturate(1.0 - fDistance * fDistance * fInvSqrRange), fPower);
	//return saturate(1 - fDistance * fDistance * fInvSqrRange);
}

#define SHADOW_CASCADES (1)
uniform mat4 g_mLightViewProj[SHADOW_CASCADES];

#define OOI_SHADOWMAP_SIZE (1024)
#define OOI_SHADOW_EPSILON (0.002)

#define OOI_SMTEXELSIZE (1.0 / OOI_SHADOWMAP_SIZE)

uniform sampler2D g_texShadowMap;

float4 EncodeFP32ToRGBA8(highp float fVal)
{
    float4 ret;
    float mult = fVal * 256.0;
    ret.x = floor(mult);
    mult = mult - ret.x;
    mult *= 256.0;
    ret.y = floor(mult);
    mult = mult - ret.y;
    mult *= 256.0;
    ret.z = floor(mult);
    mult = mult - ret.z;
    mult *= 256.0;
    ret.w = floor(mult);
    ret /= 255.0;
    return ret;
}

highp float DecodeFP32FromRGBA8(float4 vRGBA)
{
    vRGBA *= 255.0;
    return (((vRGBA.w / 256.0 + vRGBA.z) / 256.0 + vRGBA.y) / 256.0 + vRGBA.x) / 256.0;
}

float ShadowLitPercent(vec3 vPos)
{
    float4 vPosLight = float4(0.0,0.0,0.0,0.0);
	float2 ShadowTexC = float2(0.0,0.0);
	int cascadeIndex = -1;

	for(int ic=0; ic<SHADOW_CASCADES; ic++)
	{
		vPosLight = g_mLightViewProj[ic] * float4(vPos, 1);
		
		ShadowTexC = 0.5 * vPosLight.xy / vPosLight.w + float2( 0.5, 0.5 );		
		if(ShadowTexC.x > 0.0 && ShadowTexC.x < 1.0 && ShadowTexC.y > 0.0 && ShadowTexC.y < 1.0)
		{
			cascadeIndex = ic;
			break;
		}
	}
    
	float fPercentLit; 
	if(cascadeIndex < 0)
	{
    	fPercentLit = 1.0;
    }
	else
    {
        float4 vSMRgbaValue = texture2D( g_texShadowMap, ShadowTexC.xy );
        float fSMDepthValue = DecodeFP32FromRGBA8(vSMRgbaValue);
		fPercentLit = fSMDepthValue + 0.002> vPosLight.z / vPosLight.w ? 1.0 : 0.0;
    }
    
    return fPercentLit;
}

float3 LightingWithShadow(
	float3 vPos,
	float3 vNormal, 
	float3 vDiffuseAlbedo,
	float3 vSpecularAlbedo,
	float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);
	
	float4 vPosLight = float4(0.0,0.0,0.0,0.0);
	float2 ShadowTexC = float2(0.0,0.0);
	int cascadeIndex = -1;


	for(int ic=0; ic<SHADOW_CASCADES; ic++)
	{
		vPosLight = g_mLightViewProj[ic] * float4(vPos, 1);
		
		ShadowTexC = 0.5 * vPosLight.xy / vPosLight.w + float2( 0.5, 0.5 );
		//ShadowTexC.y = 1.0 - ShadowTexC.y;
		
		if(ShadowTexC.x > 0.0 && ShadowTexC.x < 1.0 && ShadowTexC.y > 0.0 && ShadowTexC.y < 1.0)
		{
			cascadeIndex = ic;
			break;
		}
	}
    
	float fPercentLit; 
	if(cascadeIndex < 0)
		fPercentLit = 1.0;
	else
    {
        float4 vSMRgbaValue = texture2D( g_texShadowMap, ShadowTexC.xy );
        float fSMDepthValue = DecodeFP32FromRGBA8(vSMRgbaValue);
		fPercentLit = fSMDepthValue + 0.002> vPosLight.z / vPosLight.w ? 1.0 : 0.0;
    }
#ifdef GL_ES

    float3 N = vNormal;
    float3 L = -DLDIR( 0 );
    float3 H = normalize( V + L );
    sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( 0 ), vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
    
#else
	int i = 0;
	//[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
		fPercentLit = 1.0;
	}
	//[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	//[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}

#endif

	return sumLit;
}


float3 Lighting(
	highp float3 vPos,
	highp float3 vNormal, 
	highp float3 vDiffuseAlbedo,
	highp float3 vSpecularAlbedo,
	highp float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);

	int i = 0;
	//[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	//[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	//[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	return sumLit;
}

#endif//LIGHTING_FXH


//Compile results:

/*
//
// Generated by Microsoft (R) HLSL Shader Compiler 9.27.952.3022
//
// Parameters:
//
//   int g_iNumDirLights;
//   int g_iNumPointLights;
//   int g_iNumSpotLights;
//
//
// Registers:
//
//   Name             Reg   Size
//   ---------------- ----- ----
//   g_iNumPointLights c0       1
//   g_iNumDirLights   c1       1
//   g_iNumSpotLights  c2       1
//
//
// Default values:
//
//   g_iNumPointLights
//     c0   = { 0, 0, 0, 0 };
//
//   g_iNumDirLights
//     c1   = { 0, 0, 0, 0 };
//
//   g_iNumSpotLights
//     c2   = { 0, 0, 0, 0 };
//

    preshader
    neg r0.x, c1.x
    add c16.x, r0.x, (1)
    add c17.x, r0.x, (2)
    add c18.x, r0.x, (3)
    neg r0.x, c0.x
    add c19.x, r0.x, (1)
    add c20.x, r0.x, (2)
    add c21.x, r0.x, (3)
    neg r0.x, c2.x
    add c22.x, r0.x, (1)
    add c23.x, r0.x, (2)
    add c24.x, r0.x, (3)

// approximately 12 instructions used
//
// Generated by Microsoft (R) HLSL Shader Compiler 9.27.952.3022
//
// Parameters:
//
//   float3 g_vCameraPos;
//   float4 g_aDirLights[8];
//   float3 g_vGlobalAmbient;
//   float4 g_aPointLights[8];
//   float4 g_aSpotLights[16];
//   int g_iNumDirLights;
//   int g_iNumPointLights;
//   int g_iNumSpotLights;
//   float4 g_tint;
//   sampler2D sampTex0;
//
//
// Registers:
//
//   Name             Reg   Size
//   ---------------- ----- ----
//   g_aSpotLights     c0      16
//   g_aPointLights    c25      8
//   g_aDirLights      c33      8
//   g_iNumPointLights c41      1
//   g_iNumDirLights   c42      1
//   g_iNumSpotLights  c43      1
//   g_vCameraPos      c44      1
//   g_vGlobalAmbient  c45      1
//   g_tint           c46      1
//   sampTex0         s0       1
//
//
// Default values:
//
//   g_aSpotLights
//     c0   = { 0, 0, 0, 0 };
//     c1   = { 0, 0, 0, 0 };
//     c2   = { 0, 0, 0, 0 };
//     c3   = { 0, 0, 0, 0 };
//     c4   = { 0, 0, 0, 0 };
//     c5   = { 0, 0, 0, 0 };
//     c6   = { 0, 0, 0, 0 };
//     c7   = { 0, 0, 0, 0 };
//     c8   = { 0, 0, 0, 0 };
//     c9   = { 0, 0, 0, 0 };
//     c10  = { 0, 0, 0, 0 };
//     c11  = { 0, 0, 0, 0 };
//     c12  = { 0, 0, 0, 0 };
//     c13  = { 0, 0, 0, 0 };
//     c14  = { 0, 0, 0, 0 };
//     c15  = { 0, 0, 0, 0 };
//
//   g_aPointLights
//     c25  = { 0, 0, 0, 0 };
//     c26  = { 0, 0, 0, 0 };
//     c27  = { 0, 0, 0, 0 };
//     c28  = { 0, 0, 0, 0 };
//     c29  = { 0, 0, 0, 0 };
//     c30  = { 0, 0, 0, 0 };
//     c31  = { 0, 0, 0, 0 };
//     c32  = { 0, 0, 0, 0 };
//
//   g_aDirLights
//     c33  = { 0, 0, 0, 0 };
//     c34  = { 0, 0, 0, 0 };
//     c35  = { 0, 0, 0, 0 };
//     c36  = { 0, 0, 0, 0 };
//     c37  = { 0, 0, 0, 0 };
//     c38  = { 0, 0, 0, 0 };
//     c39  = { 0, 0, 0, 0 };
//     c40  = { 0, 0, 0, 0 };
//
//   g_iNumPointLights
//     c41  = { 0, 0, 0, 0 };
//
//   g_iNumDirLights
//     c42  = { 0, 0, 0, 0 };
//
//   g_iNumSpotLights
//     c43  = { 0, 0, 0, 0 };
//
//   g_vCameraPos
//     c44  = { 0, 0, 0, 0 };
//
//   g_vGlobalAmbient
//     c45  = { 0, 0, 0, 0 };
//
//   g_tint
//     c46  = { 0, 0, 0, 0 };
//

    ps_3_0
    def c47, 0.300000012, 1, 0, 0
    dcl_texcoord v0.xy
    dcl_texcoord1 v1.xyz
    dcl_texcoord2 v2.xyz
    dcl_2d s0
    add r0.xyz, c28, -v2
    dp3 r0.w, r0, r0
    rsq r0.w, r0.w
    rcp r1.x, r0.w
    mul r0.xyz, r0.w, r0
    mul r0.w, r1.x, r1.x
    mov r1.y, c47.y
    mad_sat r0.w, r0.w, -c27.w, r1.y
    mul r1.xzw, r0.w, c27.xyyz
    add r2.xyz, c26, -v2
    dp3 r0.w, r2, r2
    rsq r0.w, r0.w
    rcp r2.w, r0.w
    mul r2.xyz, r0.w, r2
    mul r0.w, r2.w, r2.w
    mad_sat r0.w, r0.w, -c25.w, r1.y
    mul r3.xyz, r0.w, c25
    add r4.xyz, c44, -v2
    dp3 r0.w, r4, r4
    rsq r0.w, r0.w
    mad r5.xyz, r4, r0.w, -c34
    nrm r6.xyz, r5
    nrm r5.xyz, v1
    dp3_sat r2.w, r5, r6
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    texld r6, v0, s0
    mul r6, r6, c46
    mad r7.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c34
    mul r7.xyz, r2.w, r7
    mul r8.xyz, r6, c45
    mad r7.xyz, r7, c33, r8
    cmp r7.xyz, -c42.x, r8, r7
    mad r8.xyz, r4, r0.w, -c36
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c36
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c35, r7
    cmp r7.xyz, c16.x, r7, r8
    mad r8.xyz, r4, r0.w, -c38
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c38
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c37, r7
    cmp r7.xyz, c17.x, r7, r8
    mad r8.xyz, r4, r0.w, -c40
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c40
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c39, r7
    cmp r7.xyz, c18.x, r7, r8
    mad r8.xyz, r4, r0.w, r2
    dp3_sat r2.x, r5, r2
    nrm r9.xyz, r8
    dp3_sat r2.y, r5, r9
    mul r2.z, r2.y, r2.y
    mul r2.z, r2.z, r2.z
    mul r2.y, r2.z, r2.y
    mad r2.yzw, r2.y, c47.x, r6.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r2.xyz, r2, r3, r7
    cmp r2.xyz, -c41.x, r7, r2
    mad r3.xyz, r4, r0.w, r0
    dp3_sat r0.x, r5, r0
    nrm r7.xyz, r3
    dp3_sat r0.y, r5, r7
    mul r0.z, r0.y, r0.y
    mul r0.z, r0.z, r0.z
    mul r0.y, r0.z, r0.y
    mad r3.xyz, r0.y, c47.x, r6
    mul r0.xyz, r0.x, r3
    mad r0.xyz, r0, r1.xzww, r2
    cmp r0.xyz, c19.x, r2, r0
    add r1.xzw, c30.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    rcp r2.y, r2.x
    mul r1.xzw, r1, r2.x
    mul r2.x, r2.y, r2.y
    mad_sat r2.x, r2.x, -c29.w, r1.y
    mul r2.xyz, r2.x, c29
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c20.x, r0, r1.xzww
    add r1.xzw, c32.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    rcp r2.y, r2.x
    mul r1.xzw, r1, r2.x
    mul r2.x, r2.y, r2.y
    mad_sat r2.x, r2.x, -c31.w, r1.y
    mul r2.xyz, r2.x, c31
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c21.x, r0, r1.xzww
    add r1.xzw, c1.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c0.w, r1.y
    dp3 r2.y, -r1.xzww, c2
    add r2.y, r2.y, -c1.w
    mul_sat r2.y, r2.y, c2.w
    pow r3.x, r2.y, c3.x
    mul r2.yzw, r3.x, c0.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, -c43.x, r0, r1.xzww
    add r1.xzw, c5.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c4.w, r1.y
    dp3 r2.y, -r1.xzww, c6
    add r2.y, r2.y, -c5.w
    mul_sat r2.y, r2.y, c6.w
    pow r3.x, r2.y, c7.x
    mul r2.yzw, r3.x, c4.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c22.x, r0, r1.xzww
    add r1.xzw, c9.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c8.w, r1.y
    dp3 r2.y, -r1.xzww, c10
    add r2.y, r2.y, -c9.w
    mul_sat r2.y, r2.y, c10.w
    pow r3.x, r2.y, c11.x
    mul r2.yzw, r3.x, c8.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c23.x, r0, r1.xzww
    add r1.xzw, c13.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r1.y, r2.x, -c12.w, r1.y
    dp3 r2.x, -r1.xzww, c14
    add r2.x, r2.x, -c13.w
    mul_sat r2.x, r2.x, c14.w
    pow r3.x, r2.x, c15.x
    mul r2.xyz, r3.x, c12
    mul r2.xyz, r1.y, r2
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r0.w, r5, r1.xzww
    nrm r1.xyz, r3
    dp3_sat r1.x, r5, r1
    mul r1.y, r1.x, r1.x
    mul r1.y, r1.y, r1.y
    mul r1.x, r1.y, r1.x
    mad r1.xyz, r1.x, c47.x, r6
    mov oC0.w, r6.w
    mul r1.xyz, r0.w, r1
    mad r1.xyz, r1, r2, r0
    cmp oC0.xyz, c24.x, r0, r1

// approximately 259 instruction slots used (1 texture, 258 arithmetic)
*/
