//lighting.fxh
#ifndef __LIGHTING_FXH__
#define __LIGHTING_FXH__

/*
//Only for reference
struct PointData
{
	float4 ColorAndInvSqrRange;
	float4 PositionAndPower;
};

//Spot light calculation:
//( cos(Alpha) - AngleSub ) * AngleMul )
//CPU calc:
//AngleSub = cos(Outer / 2)
//AngleMul = 1.0f / (cos(Inner / 2) / cos(Outer / 2))
struct SpotData
{
	float4 ColorAndInvSqrRange;
	float4 PositionAndAngleSub;	
	float4 DirectionAndAngleMul;
	float4 FalloffAndPower;
};

struct DirData
{
	float4 Color;
	float4 Direction;
};
*/

#if OOI_FORCE_SIMPLE_LIGHTING

uniform float4 g_aDirLights[OOI_NUM_VEC4D_PER_DIRLIGHT];

#else

uniform float4 g_aPointLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_POINTLIGHT ];
uniform float4 g_aDirLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_DIRLIGHT ];
uniform float4 g_aSpotLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_SPOTLIGHT ];
uniform int4 g_vLightCounts;
#define g_iNumDirLights (g_vLightCounts.x)
#define g_iNumPointLights (g_vLightCounts.y)
#define g_iNumSpotLights (g_vLightCounts.z)

#endif

uniform float4x4 g_mLightViewProj[OOI_SHADOW_CASCADES];

uniform float3 g_vCameraPos;
uniform float3 g_vGlobalAmbient;

#define DLCOLOR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i ].rgb )
#define DLDIR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i + 1 ].xyz )

#if !OOI_FORCE_SIMPLE_LIGHTING

#define PLCOLOR( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].rgb )
#define PLINVSQRRANGE( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].w )
#define PLPOS( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1 ].xyz )
#define PLPOWER( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1].w )

#define SPLCOLOR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].rgb )
#define SPLINVSQRRANGE( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].w )
#define SPLPOS( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].xyz )
#define SPLANGLESUB( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].w )
#define SPLDIR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].xyz )
#define SPLANGLEMUL( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].w )
#define SPLFALLOFF( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].x )
#define SPLPOWER( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].y )

#endif

float3 BlinnPhong(float3 N, float3 L, float3 H, float3 vLightColor, float3 vDiffuse, float3 vSpecular, float3 vSpecularPower)
{
	float NL = saturate(dot(N, L));
	float NH = saturate(dot(N, H));
	return (pow( NH, vSpecularPower ) * vSpecular + vDiffuse) * NL * vLightColor;	//CHECK This.
}

//Original point attenuation function from D3D fixed function
/*
float PointAttenuation(float fRange, float3 vAttenuation, float fDistance)
{
	return step(fDistance, fRange) / (vAttenuation.x * fDistance * fDistance + vAttenuation.y * fDistance + vAttenuation.z);
}
*/

//Point attenuation function suggested by Jorge. With pow function
float PointAttenuationEx(float fInvSqrRange, float fDistance, float fPower)
{
	return pow(saturate(1 - fDistance * fDistance * fInvSqrRange), fPower);
	//return saturate(1 - fDistance * fDistance * fInvSqrRange);
}

#define OOI_SHADOWMAP_SIZE OOI_SHADOW_MAP_SIZE
#define OOI_SHADOW_EPSILON (0.003f)

#define OOI_SMTEXELSIZE (1.0f / OOI_SHADOWMAP_SIZE)

sampler2D g_texShadowMap;

void CalculateRightAndUpTexelDepthDeltas ( in float3 vShadowTexDDX,
                                           in float3 vShadowTexDDY,
                                           out float fUpTextDepthWeight,
                                           out float fRightTextDepthWeight )
{
    float2x2 matScreentoShadow = float2x2( vShadowTexDDX.xy, vShadowTexDDY.xy );
    float fDeterminant = determinant ( matScreentoShadow );
    
    float fInvDeterminant = 1.0f / fDeterminant;
    
    float2x2 matShadowToScreen = float2x2 (
        matScreentoShadow._22 * fInvDeterminant, matScreentoShadow._12 * -fInvDeterminant, 
        matScreentoShadow._21 * -fInvDeterminant, matScreentoShadow._11 * fInvDeterminant );

    float2 vRightShadowTexelLocation = float2( OOI_SMTEXELSIZE, 0.0f );
    float2 vUpShadowTexelLocation = float2( 0.0f, OOI_SMTEXELSIZE );  
    
    // Transform the right pixel by the shadow space to screen space matrix.
    float2 vRightTexelDepthRatio = mul( vRightShadowTexelLocation,  matShadowToScreen );
    float2 vUpTexelDepthRatio = mul( vUpShadowTexelLocation,  matShadowToScreen );

    // We can now caculate how much depth changes when you move up or right in the shadow map.
    // We use the ratio of change in x and y times the dervivite in X and Y of the screen space 
    // depth to calculate this change.
    fUpTextDepthWeight = 
        vUpTexelDepthRatio.x * vShadowTexDDX.z 
        + vUpTexelDepthRatio.y * vShadowTexDDY.z;
    fRightTextDepthWeight = 
        vRightTexelDepthRatio.x * vShadowTexDDX.z 
        + vRightTexelDepthRatio.y * vShadowTexDDY.z;
}

#define PCF_SAMPLE_FACTOR (1)
#define PCF_SAMPLE_DIM (PCF_SAMPLE_FACTOR + PCF_SAMPLE_FACTOR + 1)
#define PCF_TOTAL_TEX_SAMPLES (PCF_SAMPLE_DIM * PCF_SAMPLE_DIM)
#define PCF_SAMPLES (PCF_SAMPLE_FACTOR == 0 ? 1 :((PCF_SAMPLE_DIM - 1) * (PCF_SAMPLE_DIM - 1)))

void CalculatePCFPercentLit ( in int iCascadIndex,
				in float3 vShadowTexCoord, 
				in float fRightTexelDepthDelta, 
				in float fUpTexelDepthDelta, 
				out float fPercentLit)
{
    // transform to texel space
    float2 texelpos = OOI_SHADOWMAP_SIZE * vShadowTexCoord.xy;
    // Determine the lerp amounts
    float2 lerps = frac( texelpos );

	float litTable[PCF_SAMPLE_DIM][PCF_SAMPLE_DIM];

    fPercentLit = 0.0f;
    // This loop could be unrolled, and texture immediate offsets could be used if the kernel size were fixed.
    // This would be performance improvment.
    [unroll]
    for( int x = -PCF_SAMPLE_FACTOR; x <= PCF_SAMPLE_FACTOR; ++x ) 
    {
		[unroll]
        for( int y = -PCF_SAMPLE_FACTOR; y <= PCF_SAMPLE_FACTOR; ++y ) 
        {
            float depthcompare = vShadowTexCoord.z;            
            depthcompare -= OOI_SHADOW_EPSILON;
            depthcompare += fRightTexelDepthDelta * ( (float) x ) + fUpTexelDepthDelta * ( (float) y );

            // Compare the transformed pixel depth to the depth read from the map.
            float fDepthShadow = tex2D( g_texShadowMap, float2( 
                    (vShadowTexCoord.x + x * OOI_SMTEXELSIZE + iCascadIndex) / OOI_SHADOW_CASCADES, 
                    vShadowTexCoord.y + y * OOI_SMTEXELSIZE ) ).r;
		
			litTable[x+PCF_SAMPLE_FACTOR][y+PCF_SAMPLE_FACTOR] = fDepthShadow > depthcompare ? 1 : 0;
        }
    }
	
#if PCF_SAMPLE_FACTOR == 0
	fPercentLit = litTable[0][0];
#else
	for( int x = 0; x < PCF_SAMPLE_DIM - 1; x++ )
	{
		for( int y = 0; y < PCF_SAMPLE_DIM - 1; y++ )
		{
			fPercentLit += lerp(
				lerp(litTable[x][y], litTable[x+1][y], lerps.x),
				lerp(litTable[x][y+1], litTable[x+1][y+1], lerps.x), lerps.y);
		}
	}
	fPercentLit *= 1.0 / PCF_SAMPLES;
#endif
}

float3 LightingWithShadow(
	float3 vPos,
	float3 vNormal, 
	float3 vDiffuseAlbedo,
	float3 vSpecularAlbedo,
	float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);
	
	float4 vPosLight = (float4)0;
	float2 ShadowTexC = (float2)0;
	int cascadeIndex = -1;

	for(int ic=0; ic<OOI_SHADOW_CASCADES; ic++)
	{
		vPosLight = mul(float4(vPos, 1), g_mLightViewProj[ic]);
		
		ShadowTexC = 0.5 * vPosLight.xy / vPosLight.w + float2( 0.5, 0.5 );
		ShadowTexC.y = 1.0f - ShadowTexC.y;
		
		if(ShadowTexC.x > 0 && ShadowTexC.x < 1 && ShadowTexC.y > 0 && ShadowTexC.y < 1)
		{
			cascadeIndex = ic;
			break;
		}
	}
	float fPercentLit; 

	float3 vShadowMapTextureCoordViewSpace = vPosLight.xyz / vPosLight.w;
	
	float3 vShadowMapTextureCoordDDX = ddx( vShadowMapTextureCoordViewSpace );
	float3 vShadowMapTextureCoordDDY = ddy( vShadowMapTextureCoordViewSpace );
		
	float fUpTextDepthWeight, fRightTextDepthWeight;
	CalculateRightAndUpTexelDepthDeltas ( vShadowMapTextureCoordDDX, vShadowMapTextureCoordDDY, fUpTextDepthWeight, fRightTextDepthWeight );

	if(any(abs(ShadowTexC - 0.5f) > 0.5f))
		fPercentLit = 1.0f;
	else
		CalculatePCFPercentLit( cascadeIndex, float3(ShadowTexC, vShadowMapTextureCoordViewSpace.z), fRightTextDepthWeight, fUpTextDepthWeight, fPercentLit );

	if(cascadeIndex < 0)
		fPercentLit = 1.0f;
	/*
	else 
		fPercentLit = tex2D( g_texShadowMap, ShadowTexC.xy ).r + 0.002> vPosLight.z / vPosLight.w ? 1 : 0;
	*/
	
	
#if OOI_FORCE_SIMPLE_LIGHTING

	float3 N = vNormal;
	float3 L = -DLDIR( 0 );
	float3 H = normalize( V + L );
	sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( 0 ),
		vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );

#else

	int i = 0;
	[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
		fPercentLit = 1.0f;
	}
	[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}

#endif
	
	return sumLit;
}

float3 Lighting(
	float3 vPos,
	float3 vNormal, 
	float3 vDiffuseAlbedo,
	float3 vSpecularAlbedo,
	float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);
	
	
#if OOI_FORCE_SIMPLE_LIGHTING

	float3 N = vNormal;
	float3 L = -DLDIR( 0 );
	float3 H = normalize( V + L );
	sumLit += BlinnPhong( N, L, H, DLCOLOR( 0 ),
		vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );

#else
	
	int i = 0;
	[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
#endif
	
	return sumLit;
}

#endif//__LIGHTING_FXH__


//Compile results:

/*
//
// Generated by Microsoft (R) HLSL Shader Compiler 9.27.952.3022
//
// Parameters:
//
//   int g_iNumDirLights;
//   int g_iNumPointLights;
//   int g_iNumSpotLights;
//
//
// Registers:
//
//   Name             Reg   Size
//   ---------------- ----- ----
//   g_iNumPointLights c0       1
//   g_iNumDirLights   c1       1
//   g_iNumSpotLights  c2       1
//
//
// Default values:
//
//   g_iNumPointLights
//     c0   = { 0, 0, 0, 0 };
//
//   g_iNumDirLights
//     c1   = { 0, 0, 0, 0 };
//
//   g_iNumSpotLights
//     c2   = { 0, 0, 0, 0 };
//

    preshader
    neg r0.x, c1.x
    add c16.x, r0.x, (1)
    add c17.x, r0.x, (2)
    add c18.x, r0.x, (3)
    neg r0.x, c0.x
    add c19.x, r0.x, (1)
    add c20.x, r0.x, (2)
    add c21.x, r0.x, (3)
    neg r0.x, c2.x
    add c22.x, r0.x, (1)
    add c23.x, r0.x, (2)
    add c24.x, r0.x, (3)

// approximately 12 instructions used
//
// Generated by Microsoft (R) HLSL Shader Compiler 9.27.952.3022
//
// Parameters:
//
//   float3 g_vCameraPos;
//   float4 g_aDirLights[8];
//   float3 g_vGlobalAmbient;
//   float4 g_aPointLights[8];
//   float4 g_aSpotLights[16];
//   int g_iNumDirLights;
//   int g_iNumPointLights;
//   int g_iNumSpotLights;
//   float4 g_tint;
//   sampler2D sampTex0;
//
//
// Registers:
//
//   Name             Reg   Size
//   ---------------- ----- ----
//   g_aSpotLights     c0      16
//   g_aPointLights    c25      8
//   g_aDirLights      c33      8
//   g_iNumPointLights c41      1
//   g_iNumDirLights   c42      1
//   g_iNumSpotLights  c43      1
//   g_vCameraPos      c44      1
//   g_vGlobalAmbient  c45      1
//   g_tint           c46      1
//   sampTex0         s0       1
//
//
// Default values:
//
//   g_aSpotLights
//     c0   = { 0, 0, 0, 0 };
//     c1   = { 0, 0, 0, 0 };
//     c2   = { 0, 0, 0, 0 };
//     c3   = { 0, 0, 0, 0 };
//     c4   = { 0, 0, 0, 0 };
//     c5   = { 0, 0, 0, 0 };
//     c6   = { 0, 0, 0, 0 };
//     c7   = { 0, 0, 0, 0 };
//     c8   = { 0, 0, 0, 0 };
//     c9   = { 0, 0, 0, 0 };
//     c10  = { 0, 0, 0, 0 };
//     c11  = { 0, 0, 0, 0 };
//     c12  = { 0, 0, 0, 0 };
//     c13  = { 0, 0, 0, 0 };
//     c14  = { 0, 0, 0, 0 };
//     c15  = { 0, 0, 0, 0 };
//
//   g_aPointLights
//     c25  = { 0, 0, 0, 0 };
//     c26  = { 0, 0, 0, 0 };
//     c27  = { 0, 0, 0, 0 };
//     c28  = { 0, 0, 0, 0 };
//     c29  = { 0, 0, 0, 0 };
//     c30  = { 0, 0, 0, 0 };
//     c31  = { 0, 0, 0, 0 };
//     c32  = { 0, 0, 0, 0 };
//
//   g_aDirLights
//     c33  = { 0, 0, 0, 0 };
//     c34  = { 0, 0, 0, 0 };
//     c35  = { 0, 0, 0, 0 };
//     c36  = { 0, 0, 0, 0 };
//     c37  = { 0, 0, 0, 0 };
//     c38  = { 0, 0, 0, 0 };
//     c39  = { 0, 0, 0, 0 };
//     c40  = { 0, 0, 0, 0 };
//
//   g_iNumPointLights
//     c41  = { 0, 0, 0, 0 };
//
//   g_iNumDirLights
//     c42  = { 0, 0, 0, 0 };
//
//   g_iNumSpotLights
//     c43  = { 0, 0, 0, 0 };
//
//   g_vCameraPos
//     c44  = { 0, 0, 0, 0 };
//
//   g_vGlobalAmbient
//     c45  = { 0, 0, 0, 0 };
//
//   g_tint
//     c46  = { 0, 0, 0, 0 };
//

    ps_3_0
    def c47, 0.300000012, 1, 0, 0
    dcl_texcoord v0.xy
    dcl_texcoord1 v1.xyz
    dcl_texcoord2 v2.xyz
    dcl_2d s0
    add r0.xyz, c28, -v2
    dp3 r0.w, r0, r0
    rsq r0.w, r0.w
    rcp r1.x, r0.w
    mul r0.xyz, r0.w, r0
    mul r0.w, r1.x, r1.x
    mov r1.y, c47.y
    mad_sat r0.w, r0.w, -c27.w, r1.y
    mul r1.xzw, r0.w, c27.xyyz
    add r2.xyz, c26, -v2
    dp3 r0.w, r2, r2
    rsq r0.w, r0.w
    rcp r2.w, r0.w
    mul r2.xyz, r0.w, r2
    mul r0.w, r2.w, r2.w
    mad_sat r0.w, r0.w, -c25.w, r1.y
    mul r3.xyz, r0.w, c25
    add r4.xyz, c44, -v2
    dp3 r0.w, r4, r4
    rsq r0.w, r0.w
    mad r5.xyz, r4, r0.w, -c34
    nrm r6.xyz, r5
    nrm r5.xyz, v1
    dp3_sat r2.w, r5, r6
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    texld r6, v0, s0
    mul r6, r6, c46
    mad r7.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c34
    mul r7.xyz, r2.w, r7
    mul r8.xyz, r6, c45
    mad r7.xyz, r7, c33, r8
    cmp r7.xyz, -c42.x, r8, r7
    mad r8.xyz, r4, r0.w, -c36
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c36
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c35, r7
    cmp r7.xyz, c16.x, r7, r8
    mad r8.xyz, r4, r0.w, -c38
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c38
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c37, r7
    cmp r7.xyz, c17.x, r7, r8
    mad r8.xyz, r4, r0.w, -c40
    nrm r9.xyz, r8
    dp3_sat r2.w, r5, r9
    mul r3.w, r2.w, r2.w
    mul r3.w, r3.w, r3.w
    mul r2.w, r2.w, r3.w
    mad r8.xyz, r2.w, c47.x, r6
    dp3_sat r2.w, r5, -c40
    mul r8.xyz, r2.w, r8
    mad r8.xyz, r8, c39, r7
    cmp r7.xyz, c18.x, r7, r8
    mad r8.xyz, r4, r0.w, r2
    dp3_sat r2.x, r5, r2
    nrm r9.xyz, r8
    dp3_sat r2.y, r5, r9
    mul r2.z, r2.y, r2.y
    mul r2.z, r2.z, r2.z
    mul r2.y, r2.z, r2.y
    mad r2.yzw, r2.y, c47.x, r6.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r2.xyz, r2, r3, r7
    cmp r2.xyz, -c41.x, r7, r2
    mad r3.xyz, r4, r0.w, r0
    dp3_sat r0.x, r5, r0
    nrm r7.xyz, r3
    dp3_sat r0.y, r5, r7
    mul r0.z, r0.y, r0.y
    mul r0.z, r0.z, r0.z
    mul r0.y, r0.z, r0.y
    mad r3.xyz, r0.y, c47.x, r6
    mul r0.xyz, r0.x, r3
    mad r0.xyz, r0, r1.xzww, r2
    cmp r0.xyz, c19.x, r2, r0
    add r1.xzw, c30.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    rcp r2.y, r2.x
    mul r1.xzw, r1, r2.x
    mul r2.x, r2.y, r2.y
    mad_sat r2.x, r2.x, -c29.w, r1.y
    mul r2.xyz, r2.x, c29
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c20.x, r0, r1.xzww
    add r1.xzw, c32.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    rcp r2.y, r2.x
    mul r1.xzw, r1, r2.x
    mul r2.x, r2.y, r2.y
    mad_sat r2.x, r2.x, -c31.w, r1.y
    mul r2.xyz, r2.x, c31
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c21.x, r0, r1.xzww
    add r1.xzw, c1.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c0.w, r1.y
    dp3 r2.y, -r1.xzww, c2
    add r2.y, r2.y, -c1.w
    mul_sat r2.y, r2.y, c2.w
    pow r3.x, r2.y, c3.x
    mul r2.yzw, r3.x, c0.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, -c43.x, r0, r1.xzww
    add r1.xzw, c5.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c4.w, r1.y
    dp3 r2.y, -r1.xzww, c6
    add r2.y, r2.y, -c5.w
    mul_sat r2.y, r2.y, c6.w
    pow r3.x, r2.y, c7.x
    mul r2.yzw, r3.x, c4.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c22.x, r0, r1.xzww
    add r1.xzw, c9.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r2.x, r2.x, -c8.w, r1.y
    dp3 r2.y, -r1.xzww, c10
    add r2.y, r2.y, -c9.w
    mul_sat r2.y, r2.y, c10.w
    pow r3.x, r2.y, c11.x
    mul r2.yzw, r3.x, c8.xxyz
    mul r2.xyz, r2.x, r2.yzww
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r1.x, r5, r1.xzww
    nrm r7.xyz, r3
    dp3_sat r1.z, r5, r7
    mul r1.w, r1.z, r1.z
    mul r1.w, r1.w, r1.w
    mul r1.z, r1.w, r1.z
    mad r3.xyz, r1.z, c47.x, r6
    mul r1.xzw, r1.x, r3.xyyz
    mad r1.xzw, r1, r2.xyyz, r0.xyyz
    cmp r0.xyz, c23.x, r0, r1.xzww
    add r1.xzw, c13.xyyz, -v2.xyyz
    dp3 r2.x, r1.xzww, r1.xzww
    rsq r2.x, r2.x
    mul r1.xzw, r1, r2.x
    rcp r2.x, r2.x
    mul r2.x, r2.x, r2.x
    mad_sat r1.y, r2.x, -c12.w, r1.y
    dp3 r2.x, -r1.xzww, c14
    add r2.x, r2.x, -c13.w
    mul_sat r2.x, r2.x, c14.w
    pow r3.x, r2.x, c15.x
    mul r2.xyz, r3.x, c12
    mul r2.xyz, r1.y, r2
    mad r3.xyz, r4, r0.w, r1.xzww
    dp3_sat r0.w, r5, r1.xzww
    nrm r1.xyz, r3
    dp3_sat r1.x, r5, r1
    mul r1.y, r1.x, r1.x
    mul r1.y, r1.y, r1.y
    mul r1.x, r1.y, r1.x
    mad r1.xyz, r1.x, c47.x, r6
    mov oC0.w, r6.w
    mul r1.xyz, r0.w, r1
    mad r1.xyz, r1, r2, r0
    cmp oC0.xyz, c24.x, r0, r1

// approximately 259 instruction slots used (1 texture, 258 arithmetic)
*/