#include "lighting.fxh"

float4x4 	g_mWVP;
float4x4 	g_mWorld;

float3 		g_vSpecularAlbedo;
float		g_fSpecularPower;

struct VS_IN
{
	float4 a_Position 	: POSITION;
	float4 a_Color 		: COLOR0;
	float3 a_Normal : NORMAL0;
};

struct VS_OUTPUT
{
    float4 v_position   	: POSITION;   // vertex position 
	float4 v_color   		: COLOR;
	float3 v_positionWS 	: TEXCOORD0;
	float3 v_normalWS		: TEXCOORD1;
};

VS_OUTPUT RenderSceneVS( VS_IN In )
{
    VS_OUTPUT Output;
   
	Output.v_position = mul(In.a_Position, g_mWVP);
    Output.v_color = In.a_Color;
	
	Output.v_positionWS = mul( In.a_Position, g_mWorld ).xyz;
	Output.v_normalWS = mul( In.a_Normal, (float3x3)g_mWorld );

    return Output;    
}

struct PS_IN
{
	float4 v_color  : COLOR;
	float3 v_positionWS 	: TEXCOORD0;
	float3 v_normalWS		: TEXCOORD1;
};

float4 RenderScenePS( PS_IN In ) : COLOR
{ 
//    return In.v_color;
	return float4( Lighting(
		In.v_positionWS,
		normalize(In.v_normalWS),
		In.v_color.rgb,
		g_vSpecularAlbedo,
		g_fSpecularPower), In.v_color.a);
}

technique technique_main
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderSceneVS();
        PixelShader  = compile ps_3_0 RenderScenePS(); 
    }
}

