#include "lighting.fxh"

float4x4 	g_mWVP;
float4x4 	g_mWorld;
float 		g_fTime;

float3 		g_vSpecularAlbedo;
float		g_fSpecularPower;

sampler2D g_texDiffuse;
sampler2D g_texNormal;

struct VS_IN
{
	float4 a_Position 	: POSITION;
	float3 a_Normal 	: NORMAL0;
	float2 a_UV1 		: TEXCOORD0;
	float3 a_Tangent 	: TANGENT0;
};

struct VS_OUTPUT
{
    float4 v_position   	: POSITION;   // vertex position 
	float2 v_uv 			: TEXCOORD0;
	float3 v_positionWS 	: TEXCOORD1;
	float3 v_normalWS		: TEXCOORD2;
	float3 v_tangentWS		: TEXCOORD3;
};

VS_OUTPUT vs_main( VS_IN In )
{
    VS_OUTPUT Output;
   
	Output.v_position = mul(In.a_Position, g_mWVP);
    Output.v_uv = In.a_UV1;

	Output.v_positionWS = mul( In.a_Position, g_mWorld ).xyz;
	Output.v_normalWS = mul( In.a_Normal, (float3x3)g_mWorld );
	Output.v_tangentWS = mul( In.a_Tangent, (float3x3)g_mWorld );

    return Output;    
}

struct PS_IN
{
	float2 v_uv 		: TEXCOORD0;
	float3 v_positionWS : TEXCOORD1;
	float3 v_normalWS	: TEXCOORD2;
	float3 v_tangentWS	: TEXCOORD3;
};

float4 ps_main( PS_IN In ) : COLOR
{ 
    float4 vDiffuse = tex2D(g_texDiffuse, In.v_uv);
	float3 vNrmMapVal = tex2D(g_texNormal, In.v_uv * 10 + float2(g_fTime, 0)).xyz * 2.0f - 1.0f;

	float3 vNormalWS = normalize(In.v_normalWS);
	float3 vTangentWS = normalize(In.v_tangentWS);
	float3 vBitangentWS = normalize(cross(vNormalWS, vTangentWS));
	
	float3x3 matTangent = float3x3(vTangentWS, vBitangentWS, vNormalWS);
	vNrmMapVal = mul(vNrmMapVal, matTangent);
	
	return float4( Lighting(
		In.v_positionWS,
		vNrmMapVal,
		vDiffuse.rgb,
		g_vSpecularAlbedo,
		g_fSpecularPower), vDiffuse.a);
}

technique technique_main
{
    pass P0
    {          
        VertexShader = compile vs_3_0 vs_main();
        PixelShader  = compile ps_3_0 ps_main(); 
    }
}