#include "lighting.fxh"

float4x4 	g_mWVP;
float4x4 	g_mWorld;

float3 		g_vSpecularAlbedo;
float		g_fSpecularPower;
float3      g_vTint;

sampler2D 	g_texture0;

struct VS_IN
{
	float4 a_Position 	: POSITION;
	float3 a_Normal 	: NORMAL0;
	float2 a_UV1 		: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 v_position   	: POSITION;   // vertex position 
	float2 v_uv 			: TEXCOORD0;
	float3 v_positionWS 	: TEXCOORD1;
	float3 v_normalWS		: TEXCOORD2;
};

VS_OUTPUT vs_main( VS_IN In )
{
    VS_OUTPUT Output;
   
	Output.v_position = mul(In.a_Position, g_mWVP);
    Output.v_uv = In.a_UV1;

	Output.v_positionWS = mul( In.a_Position, g_mWorld ).xyz;
	Output.v_normalWS = mul( In.a_Normal, (float3x3)g_mWorld );

    return Output;    
}

struct PS_IN
{
	float2 v_uv 	: TEXCOORD0;
	float3 v_positionWS 	: TEXCOORD1;
	float3 v_normalWS		: TEXCOORD2;
};

float4 ps_main( PS_IN In ) : COLOR
{ 
    float4 vDiffuse = tex2D(g_texture0,In.v_uv);
	return float4( g_vTint * Lighting(
		In.v_positionWS,
		normalize(In.v_normalWS),
		vDiffuse.rgb,
		g_vSpecularAlbedo,
		g_fSpecularPower), vDiffuse.a);
}