#ifndef OOI_SHADER_CONFIG_H
#define OOI_SHADER_CONFIG_H

#define OOI_MAX_DYNAMIC_LIGHTS				(1) //Shader-Config//
#define OOI_NUM_VEC4D_PER_DIRLIGHT			(2) //Shader-Config//
#define OOI_NUM_VEC4D_PER_POINTLIGHT		(2) //Shader-Config//
#define OOI_NUM_VEC4D_PER_SPOTLIGHT			(4) //Shader-Config//
#define OOI_SHADOW_MAX_CASCADES	(1)
#define OOI_SHADOW_MAP_SIZE	(2048)
#ifndef GL_ES    //Shader-Config//
#define highp   //Shader-Config//
#define lowp    //Shader-Config//
#define mediump //Shader-Config//
#endif          //Shader-Config//

#define OOI_PER_VIEW_CBSLOT b0
#define OOI_PER_OBJECT_CBSLOT b1
#define OOI_LIGHTING_CBSLOT b2
#define OOI_PER_MATERIAL_CBSLOT b3
#define OOI_PER_INSTANCE_CBSLOT b4
#define OOI_SKINNING_CBSLOT b5

cbuffer SharkPerView : register(OOI_PER_VIEW_CBSLOT)
{
    float4x4 g_mView : packoffset(c0);
    float4x4 g_mViewProjection : packoffset(c4);
    float3 g_vCameraPos : packoffset(c8.x);
    float g_fTime : packoffset(c8.w);
    float4 g_vGlobalAmbient : packoffset(c10);
    float4x4 g_mLightViewProj[OOI_SHADOW_MAX_CASCADES] : packoffset(c11);
};

cbuffer SharkPerObject : register(OOI_PER_OBJECT_CBSLOT)
{
    	float4x4 g_mWorld;
    	float4x4 g_mWorldView;
    	float4x4 g_mWVP;
    	float3x4 g_mWorldIT;
};

cbuffer SharkLighting : register(OOI_LIGHTING_CBSLOT)
{
     float4 g_aDirLightParams[OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_DIRLIGHT];
     float4 g_aPointLightParams[OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_POINTLIGHT];
     float4 g_aSpotLightParams[OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_SPOTLIGHT];
     int4 g_iLightCounts;
};

cbuffer SharkSkinning : register(OOI_SKINNING_CBSLOT)
{
    float4x4 u_SkinningMatrix[64];
};

#define DefinePerMaterialCB cbuffer SharkPerMaterial : register(OOI_PER_MATERIAL_CBSLOT)
#define DefinePerInstanceCB cbuffer SharkPerInstance : register(OOI_PER_INSTANCE_CBSLOT)

SamplerState SamplerPointWrap : register(s0);
SamplerState SamplerPointClamp : register(s1);
SamplerState SamplerPointMirror : register(s2);
SamplerState SamplerBilinearWrap : register(s3);
SamplerState SamplerBilinearClamp : register(s4);
SamplerState SamplerBilinearMirror : register(s5);
SamplerState SamplerTrilinearWrap : register(s6);
SamplerState SamplerTrilinearClamp : register(s7);
SamplerState SamplerTrilinearMirror : register(s8);
SamplerComparisonState SamplerPointShadowComparison : register(s9);
SamplerComparisonState SamplerBilinearShadowComparison : register(s10);


#ifdef OOI_SHADER_USING_CG
#define OOI_mul( v, mat ) mul(mat, v)
#else
#define OOI_mul( v, mat ) mul(v, mat)
#endif//OOI_SHADER_USING_CG

#endif//OOI_SHADER_CONFIG_H