//lighting.fxh
#ifndef __LIGHTING_FXH__
#define __LIGHTING_FXH__

#include "SharkCommonDX11.fxh"

#define SHADOW_CASCADES OOI_SHADOW_CASCADES

#define g_iNumDirLights (g_vLightCounts.x)
#define g_iNumPointLights (g_vLightCounts.y)
#define g_iNumSpotLights (g_vLightCounts.z)

#define PLCOLOR( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].rgb )
#define PLINVSQRRANGE( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i ].w )
#define PLPOS( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1 ].xyz )
#define PLPOWER( i ) ( g_aPointLights[ OOI_NUM_VEC4D_PER_POINTLIGHT * i + 1].w )


#define DLCOLOR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i ].rgb )
#define DLDIR( i ) ( g_aDirLights[ OOI_NUM_VEC4D_PER_DIRLIGHT * i + 1 ].xyz )

#define SPLCOLOR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].rgb )
#define SPLINVSQRRANGE( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i ].w )
#define SPLPOS( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].xyz )
#define SPLANGLESUB( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 1 ].w )
#define SPLDIR( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].xyz )
#define SPLANGLEMUL( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 2 ].w )
#define SPLFALLOFF( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].x )
#define SPLPOWER( i ) ( g_aSpotLights[ OOI_NUM_VEC4D_PER_SPOTLIGHT * i + 3 ].y )


float3 BlinnPhong(float3 N, float3 L, float3 H, float3 vLightColor, float3 vDiffuse, float3 vSpecular, float3 vSpecularPower)
{
	float NL = saturate(dot(N, L));
	float NH = saturate(dot(N, H));
	return ( /*pow( NH, vSpecularPower ) * vSpecular + */ vDiffuse) * NL * vLightColor;	//CHECK This.
}

//Original point attenuation function from D3D fixed function
/*
float PointAttenuation(float fRange, float3 vAttenuation, float fDistance)
{
	return step(fDistance, fRange) / (vAttenuation.x * fDistance * fDistance + vAttenuation.y * fDistance + vAttenuation.z);
}
*/

//Point attenuation function suggested by Jorge. With pow function
float PointAttenuationEx(float fInvSqrRange, float fDistance, float fPower)
{
	return pow(saturate(1 - fDistance * fDistance * fInvSqrRange), fPower);
	//return saturate(1 - fDistance * fDistance * fInvSqrRange);
}

#define OOI_SHADOWMAP_SIZE OOI_SHADOW_MAP_SIZE
#define OOI_SHADOW_EPSILON (0.003f)

#define OOI_SMTEXELSIZE (1.0f / OOI_SHADOWMAP_SIZE)

Texture2D g_ShadowMapTexture;
SamplerState ColorSampler : register(s0);

#define PCF_SAMPLE_DIM (1)

#if SHADER_MODEL > 20

void CalculatePCFPercentLit ( in int iCascadIndex,
				in float3 vShadowTexCoord,
				out float fPercentLit)
{
	float depthcompare = vShadowTexCoord.z;            
	depthcompare -= OOI_SHADOW_EPSILON;
	vShadowTexCoord.x = (vShadowTexCoord.x + iCascadIndex) / SHADOW_CASCADES;
	
	fPercentLit = 0;
	for(int i=0; i<PCF_SAMPLE_DIM; i++)
	{
		for(int j=0; j<PCF_SAMPLE_DIM; j++)
		{
			fPercentLit += g_ShadowMapTexture.SampleCmpLevelZero(
				SamplerBilinearShadowComparison, 
				vShadowTexCoord.xy,
				depthcompare,
				int2(i - PCF_SAMPLE_DIM / 2, j - PCF_SAMPLE_DIM / 2));
		}
	}
	fPercentLit /= PCF_SAMPLE_DIM * PCF_SAMPLE_DIM;
}

//Assume that:
//-We have only one cascade
//-All "rendered" pixel on the shadow map are effective shadowing pixels

float3 ComputeLightUVZ(float3 vPosWS, int iCascadeIdx)
{
	float4 lightSpaceCoord = mul( float4(vPosWS, 1), g_mLightViewProj[iCascadeIdx] );
	float3 ret = lightSpaceCoord.xyz / lightSpaceCoord.w;
	ret.z = min(ret.z, 1.0f);
	ret.xy = float2(0.5, -0.5f) * ret.xy + float2( 0.5, 0.5 );
	return ret;
}

float CalculateShadowSimple(float3 lightSpaceUVZCascade0)
{
	float2 ShadowTexC = lightSpaceUVZCascade0.xy;
	if(any(abs(ShadowTexC - 0.5f) > 0.5f))
	{
		return 1.0f;
	}
	else
	{
		return g_ShadowMapTexture.SampleCmpLevelZero(SamplerBilinearShadowComparison, ShadowTexC, lightSpaceUVZCascade0.z - OOI_SHADOW_EPSILON).r;
	}
}


float CalculateShadowSimpleTwoStage(float3 lightSpaceUVZCascade0, float3 lightSpaceUVZCascade1)
{
	float ret = 1.0f;
	float2 ShadowTexC = 0.5 * lightSpaceUVZCascade0.xy + float2( 0.5, 0.5 );
	if(all(abs(ShadowTexC - 0.5f) < 0.5f))
	{
		ShadowTexC.y = 1.0f - ShadowTexC.y;
		ShadowTexC.x = ShadowTexC.x * 0.5f;
		float fShadow = g_ShadowMapTexture.SampleCmpLevelZero(SamplerBilinearShadowComparison, ShadowTexC, lightSpaceUVZCascade0.z - OOI_SHADOW_EPSILON).r;
		ret = min(ret, fShadow);
	}
	
	ShadowTexC = 0.5 * lightSpaceUVZCascade1.xy + float2(0.5f, 0.5f);
	if(all(abs(ShadowTexC - 0.5f) < 0.5f))
	{
		ShadowTexC.y = 1.0f - ShadowTexC.y;
		ShadowTexC.x = ShadowTexC.x * 0.5f + 0.5f;
		float fShadow = g_ShadowMapTexture.SampleCmpLevelZero(SamplerBilinearShadowComparison, ShadowTexC, lightSpaceUVZCascade1.z - OOI_SHADOW_EPSILON).r;
		ret = min(ret, fShadow);
	}
	return ret;
}

float CalculateShadowPercentLit(float3 vPos)
{
	float4 vPosLight = (float4)0;
	float2 ShadowTexC = (float2)0;
	int cascadeIndex = -1;

	for(int ic=0; ic<SHADOW_CASCADES; ic++)
	{
		vPosLight = mul(float4(vPos, 1), g_mLightViewProj[ic]);
		
		ShadowTexC = 0.5 * vPosLight.xy / vPosLight.w + float2( 0.5, 0.5 );
		ShadowTexC.y = 1.0f - ShadowTexC.y;
		
		if(ShadowTexC.x > 0 && ShadowTexC.x < 1 && ShadowTexC.y > 0 && ShadowTexC.y < 1)
		{
			cascadeIndex = ic;
			break;
		}
	}
	float fPercentLit = 1.0f;
	
	float depthToCompare = vPosLight.z / vPosLight.w;
	depthToCompare = min(1.0f, depthToCompare);

	if(any(abs(ShadowTexC - 0.5f) > 0.5f))
		fPercentLit = 1.0f;
	else
		CalculatePCFPercentLit( cascadeIndex, float3(ShadowTexC, depthToCompare), fPercentLit );	//fRightTextDepthWeight, fUpTextDepthWeight, 

	if(cascadeIndex < 0)
		fPercentLit = 1.0f;
	return fPercentLit;
}

float3 LightingWithShadow(
	float3 vPos,
	float3 vNormal, 
	float3 vDiffuseAlbedo,
	float3 vSpecularAlbedo,
	float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient.rgb * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);
	
	float fPercentLit = CalculateShadowPercentLit(vPos);

#if OOI_FORCE_SIMPLE_LIGHTING
	
	float3 N = vNormal;
	float3 L = -DLDIR( 0 );
	float3 H = normalize( V + L );
	sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( 0 ),
		vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
		
#else

	int i = 0;
	[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += fPercentLit * BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
		fPercentLit = 1.0f;
	}
	[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
#endif
	
	return sumLit;
}

#endif //SHADER_MODEL < 30

float3 Lighting(
	float3 vPos,
	float3 vNormal, 
	float3 vDiffuseAlbedo,
	float3 vSpecularAlbedo,
	float vSpecularPower)
{
	float3 sumLit = g_vGlobalAmbient.rgb * vDiffuseAlbedo;
	float3 V = normalize(g_vCameraPos - vPos);
	
	vNormal = normalize(vNormal);

#if OOI_FORCE_SIMPLE_LIGHTING
	
	float3 N = vNormal;
	float3 L = -DLDIR( 0 );
	float3 H = normalize( V + L );
	sumLit += BlinnPhong( N, L, H, DLCOLOR( 0 ),
		vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
		
#else

	int i = 0;
	[unroll] //compiler unroll this with count inferred from array length.
	for(i=0; i<g_iNumDirLights; i++)	//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 L = -DLDIR( i );
		float3 H = normalize( V + L );
		sumLit += BlinnPhong( N, L, H, DLCOLOR( i ),
			vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	[unroll]
	for(i=0; i<g_iNumPointLights; i++)//OOI_MAX_DYNAMIC_LIGHTS
	{
		float3 N = vNormal;
		float3 vObjToLight = PLPOS(i) - vPos;//
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float3 vLightColor = PLCOLOR(i) * PointAttenuationEx(PLINVSQRRANGE(i), fDistance, PLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
	[unroll]
	for(i=0; i<g_iNumSpotLights; i++)
	{
		float3 N = vNormal;
		float3 vObjToLight = SPLPOS(i) - vPos;
		float fDistance = length(vObjToLight);
		float3 L = vObjToLight / fDistance;
		float3 H = normalize( V + L );
		float fAngleFactor = (dot(-L, SPLDIR(i)) - SPLANGLESUB(i)) * SPLANGLEMUL(i);
		float fSpotFactor = pow(saturate(fAngleFactor), SPLFALLOFF(i));
		float3 vLightColor = fSpotFactor * SPLCOLOR(i) * PointAttenuationEx( SPLINVSQRRANGE(i), fDistance, SPLPOWER(i));
		sumLit += BlinnPhong( N, L, H, vLightColor, vDiffuseAlbedo, vSpecularAlbedo, vSpecularPower );
	}
	
#endif
	
	return sumLit;
}

#endif//__LIGHTING_FXH__