#include "shark/effects/Lighting.fxh"

uniform float4x4 	g_mWVP;
uniform float4x4 	g_mWorld;
uniform float4x4	u_SkinningMatrix[50];

sampler2D g_texture0;

struct VS_IN
{
	float4 a_Position 	: POSITION;
	float4 a_Color 		: COLOR0;
	float3 a_Normal 	: NORMAL0;
	float2 a_UV1 		: TEXCOORD0;
	int4 a_BoneIndices : BLENDINDICES0;
	float4 a_BoneWeight : BLENDWEIGHT0;
};

struct VS_OUTPUT
{
    float4 v_position   	: POSITION;
	float2 v_uv 			: TEXCOORD0;
	float3 v_positionWS 	: TEXCOORD1;
	float3 v_normalWS		: TEXCOORD2;
};

VS_OUTPUT vs_main(
	float4 a_Position 	: POSITION,
	float4 a_Color 		: COLOR,
	float3 a_Normal 	: NORMAL,
	float2 a_UV1 		: TEXCOORD0,
	int4 a_BoneIndices : BLENDINDICES,
	float4 a_BoneWeight : BLENDWEIGHT)
{
    VS_OUTPUT Output;
   
   	float4 p1 = OOI_mul( a_Position, u_SkinningMatrix[a_BoneIndices.x] );
	float4 p2 = OOI_mul( a_Position, u_SkinningMatrix[a_BoneIndices.y] );
	float4 p = p1 * a_BoneWeight.x + p2 * a_BoneWeight.y;
	Output.v_position = OOI_mul( p, g_mWVP );
    Output.v_uv = a_UV1;
	Output.v_positionWS = OOI_mul( a_Position, g_mWorld ).xyz;
	
   	float3 n1 = OOI_mul( a_Normal, (float3x3)u_SkinningMatrix[a_BoneIndices.x] );
	float3 n2 = OOI_mul( a_Normal, (float3x3)u_SkinningMatrix[a_BoneIndices.y] );
	float3 n = n1 * a_BoneWeight.x + n2 * a_BoneWeight.y;
	Output.v_normalWS = OOI_mul( n, (float3x3)g_mWorld );
	
    return Output;    
}

float4 ps_main( VS_OUTPUT In ) : COLOR
{ 
	float4 vDiffuse = tex2D(g_texture0, float2(In.v_uv.x, 1.0f - In.v_uv.y));
	
	return float4( Lighting(
		In.v_positionWS,
		normalize(In.v_normalWS),
		vDiffuse.rgb,
		float3(0.5,0.5,0.5),
		20.0f), vDiffuse.a);
}

technique technique_main
{
    pass P0
    {          
        VertexShader = compile vs_3_0 vs_main();
        PixelShader  = compile ps_3_0 ps_main(); 
    }
}
