#ifndef __SHARKCOMMONDX11_H__
#define __SHARKCOMMONDX11_H__

#define OOI_PER_VIEW_CBSLOT b0
#define OOI_PER_OBJECT_CBSLOT b1
#define OOI_LIGHTING_CBSLOT b2
#define OOI_PER_MATERIAL_CBSLOT b3
#define OOI_PER_INSTANCE_CBSLOT b4
#define OOI_SKINNING_CBSLOT b5
#define OOI_INSTANCING_CBSLOT b6

#define OOI_INSTANCING_MAX_OBJ (64)

cbuffer SharkPerView : register(OOI_PER_VIEW_CBSLOT)
{
    float4x4 g_mView : packoffset(c0);
    float4x4 g_mViewProjection : packoffset(c4);
    float3 g_vCameraPos : packoffset(c8.x);
    float g_fTime : packoffset(c8.w);
    float4 g_vViewportSize : packoffset(c9);
    float4 g_vGlobalAmbient : packoffset(c10);
	float4 g_vFogColor : packoffset(c11);
	float4 g_vFogDistanceFactors : packoffset(c12);
	float4x4 g_mViewInverse : packoffset(c13);
    float4x4 g_mLightViewProj[OOI_SHADOW_CASCADES] : packoffset(c17);
};

#ifndef OOI_DISABLE_FOG
float FOG_FACTOR(float z)
{
	return smoothstep(g_vFogDistanceFactors.x, g_vFogDistanceFactors.y, z);
}
#define OUTPUT_PIXEL_FOGGED(c, t)	(lerp((c), g_vFogColor, t))
#else // OOI_DISABLE_FOG
#define OUTPUT_PIXEL_FOGGED(c, t)	(c)
#endif // OOI_DISABLE_FOG

cbuffer SharkPerObject : register(OOI_PER_OBJECT_CBSLOT)
{
	float4x4 g_mWorld;
	float4x4 g_mWorldView;
	float4x4 g_mWVP;
	float3x4 g_mWorldIT;
};

cbuffer SharkLighting : register(OOI_LIGHTING_CBSLOT)
{
#if OOI_FORCE_SIMPLE_LIGHTING
	float4 g_aDirLights[OOI_NUM_VEC4D_PER_DIRLIGHT];
#else
	float4 g_aDirLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_DIRLIGHT ];
	float4 g_aPointLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_POINTLIGHT ];
	float4 g_aSpotLights[ OOI_MAX_DYNAMIC_LIGHTS * OOI_NUM_VEC4D_PER_SPOTLIGHT ];
	int4 g_vLightCounts;
#endif
};

cbuffer SharkSkinning : register(OOI_SKINNING_CBSLOT)
{
    float4x4 u_SkinningMatrix[32];
};

cbuffer SharkInstancing : register(OOI_INSTANCING_CBSLOT)
{
	float4x4 g_aInstancedWorldMatrix[OOI_INSTANCING_MAX_OBJ];
};

#define DefinePerMaterialCB cbuffer SharkPerMaterial : register(OOI_PER_MATERIAL_CBSLOT)
#define DefinePerInstanceCB cbuffer SharkPerInstance : register(OOI_PER_INSTANCE_CBSLOT)

SamplerState SamplerPointWrap : register(s0);
SamplerState SamplerPointClamp : register(s1);
SamplerState SamplerPointMirror : register(s2);
SamplerState SamplerBilinearWrap : register(s3);
SamplerState SamplerBilinearClamp : register(s4);
SamplerState SamplerBilinearMirror : register(s5);
SamplerState SamplerTrilinearWrap : register(s6);
SamplerState SamplerTrilinearClamp : register(s7);
SamplerState SamplerTrilinearMirror : register(s8);
SamplerComparisonState SamplerPointShadowComparison : register(s9);
SamplerComparisonState SamplerBilinearShadowComparison : register(s10);

#endif//__SHARKCOMMONDX11_H__